/* parser.ml :: mbc parser :: jec2160 */
%{ open Ast %}

/* tokens */
%token LPAREN RPAREN COLON COMMA
%token PLUS MINUS TIMES DIVIDE
%token EQ NEQ LT GT LEQ GEQ
%token IF THEN ELSE FOR TO STEP NEXT
%token WHILE DO LOOP GOTO
%token GOSUB RETURN END PRINT INPUT
%token EOF
%token <int> LITERAL
%token <string> ID

/* associativity and precedence */
%nonassoc ELSE
%left EQ NEQ
%left GT GEQ LT LEQ
%left PLUS MINUS
%left TIMES DIVIDE

/* change start to stmt after implemented */
%start expr
%type < Ast.expr> expr

%%

expr:
  | ID                          { Id($1) } 
  | expr   PLUS   expr          { Binop($1, Add, $3, $3) }
  | expr   MINUS  expr          { Binop($1, Sub, $3, $3) }
  | expr   TIMES  expr          { Binop($1, Mul, $3, $3) }
  | expr   DIVIDE expr          { Binop($1, Div, $3, $3) }
  | expr   EQ     expr          { Binop($1, Equal, $3, $3) }
  | expr   NEQ    expr          { Binop($1, Notequal, $3, $3) }
  | expr   LT     expr	        { Binop($1, Less, $3, $3) }
  | expr   GT     expr          { Binop($1, Greater, $3, $3) }
  | expr   LEQ    expr          { Binop($1, Leq, $3, $3) }
  | expr   GEQ    expr          { Binop($1, Geq, $3, $3) }
  | LPAREN expr RPAREN          { $2 }
  | LITERAL                     { Lit($1) }
  | IF expr THEN expr ELSE expr { Binop($2, Ifthenelse, $4, $6) }
  | IF expr THEN expr           { Binop($2, Ifthen, $4, $4) }
  | WHILE expr DO expr LOOP     { Binop($2, Whileop, $4, $4) }
  | FOR expr TO expr expr NEXT  { Binop($2, Forop, $4, $5) }
  | PRINT expr                  { Binop($2, Printop, $2, $2) }
  | EOF							{ Endop }
  | END							{ Endop }




