(* scanner.mll :: mbc scanner :: jec2160 *)
{ open Parser }

let whitespace = [' ' '\t' '\r' '\n']

rule token = parse
  whitespace { token lexbuf }
| "REM"      { comment lexbuf }
| '('        { LPAREN }
| ')'        { RPAREN }
| ':'        { COLON }
| ','        { COMMA }
| '+'        { PLUS }
| '-'        { MINUS }
| '*'        { TIMES }
| '/'        { DIVIDE }
| '='        { EQ }
| '<'        { LT }
| '>'        { GT }
| "<>"       { NEQ }
| "<="       { LEQ }
| ">="       { GEQ }
| "if"       { IF }
| "then"     { THEN }
| "else"     { ELSE }
| "for"      { FOR }
| "to"       { TO }
| "step"     { STEP }
| "next"     { NEXT }
| "while"    { WHILE }
| "do"       { DO }
| "loop"     { LOOP }
| "goto"     { GOTO }
| "gosub"    { GOSUB }
| "return"   { RETURN }
| "end"      { END }
| "print"    { PRINT }
| "input"    { INPUT }

(* literals and identifiers               *)
| ['0'-'9']+ as lit              { LITERAL(int_of_string lit) }
| ['a'-'z' 'A'-'Z']+ as lxm      { ID(lxm) }

(* extra                                  *)
| eof       { EOF }
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) }

(* comment handler                        *)
and comment = parse
  '\n'      { token lexbuf }
| _         { comment lexbuf }


